﻿using System.Text;


namespace HeuristicLab.Problems.ProgramSynthesis.Push.Stack {
  using System.Collections;
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Constants;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Extensions;

  /// <summary>
  /// Optimized for string concatenation to reduce memory effort
  /// </summary>
  public class PrintStack : IPrintStack {
    private int lineCount = 0;
    private readonly StringBuilder stringBuilder = new StringBuilder();

    IEnumerator IEnumerable.GetEnumerator() {
      var lines = AsStrings();
      return lines.GetEnumerator();
    }

    public void Add(string item) {
      Push(item);
    }

    void IPushStack.Clear() {
      stringBuilder.Clear();
    }

    public void NewLine() {
      stringBuilder.Append(PushEnvironment.NewLine);
      lineCount++;
    }

    public void Push<T>(T item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Append(item);
    }

    public void Push(float item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Concat(item);
    }

    public void Push(double item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Concat(item);
    }

    public void Push(long item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Concat(item);
    }

    public void Push(int item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Concat(item);
    }

    public void Push(char item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Append(item);
    }

    public void Push(bool item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Append(item);
    }


    public void Push(string item) {
      if (IsEmpty) lineCount = 1;
      stringBuilder.Append(item);
    }

    public void Push(IReadOnlyList<string> items, int startIndex) {
      for (var i = startIndex; i < items.Count; i++)
        stringBuilder.Append(items[i]);
    }

    public void Push(IReadOnlyList<string> items) {
      Push(items, 0);
    }

    public void Push(IEnumerable<string> items) {
      foreach (var item in items)
        stringBuilder.Append(item);
    }

    public IEnumerable<string> AsStrings() {
      var lines = stringBuilder.ToString().Split(PushEnvironment.NewLine);
      return lines;
    }

    public IEnumerable<object> AsObjects() {
      return AsStrings();
    }

    int IPushStack.Count
    {
      get
      {
        return lineCount;
      }
    }

    public bool IsEmpty { get { return stringBuilder.Length == 0; } }

    public bool IsEnabled { get; set; }

    public bool IsReadOnly { get { return false; } }

    public override string ToString() {
      return stringBuilder.ToString();
    }
  }
}
